@@ -20,20 +20,6 @@ module Sequel
         :sqlrelay => "SQLRelay"
       }
       
-      def initialize(opts)
-        super
-        case @opts[:db_type]
-        when 'mssql'
-          Sequel.ts_require 'adapters/shared/mssql'
-          extend Sequel::MSSQL::DatabaseMethods
-          def self.dataset(*args)
-            ds = super
-            ds.extend Sequel::MSSQL::DatasetMethods
-            ds
-          end
-        end
-      end
-
       # Converts a uri to an options hash. These options are then passed
       # to a newly created database object.
       def self.uri_to_options(uri) # :nodoc:
@@ -62,8 +48,12 @@ module Sequel
         end
         ::DBI.connect(dbname, opts[:user], opts[:password])
       end
+
+      def disconnect_connection(c)
+        c.disconnect
+      end
       
-      def execute(sql, opts={})
+      def execute(sql, opts=OPTS)
         synchronize(opts[:server]) do |conn|
           r = log_yield(sql){conn.execute(sql)}
           yield(r) if block_given?
@@ -71,15 +61,19 @@ module Sequel
         end
       end
       
-      def do(sql, opts={})
+      def execute_dui(sql, opts=OPTS)
         synchronize(opts[:server]){|conn| log_yield(sql){conn.do(sql)}}
       end
-      alias_method :execute_dui, :do
 
       private
 
-      def disconnect_connection(c)
-        c.disconnect
+      def adapter_initialize
+        case @opts[:db_type]
+        when 'mssql'
+          Sequel.require 'adapters/shared/mssql'
+          extend Sequel::MSSQL::DatabaseMethods
+          extend_datasets Sequel::MSSQL::DatasetMethods
+        end
       end
     end
     
@@ -90,16 +84,10 @@ module Sequel
         execute(sql) do |s|
           begin
             columns = cols = s.column_names.map{|c| output_identifier(c)}
-            if opts[:offset] && offset_returns_row_number_column?
-              rn = row_number_column
-              columns = columns.dup
-              columns.delete(rn)
-            end
             @columns = columns
             s.fetch do |r|
               row = {}
               cols.each{|c| row[c] = r.shift}
-              row.delete(rn) if rn
               yield row
             end
           ensure
