@@ -9,25 +9,6 @@ module Sequel
       DRV_NAME_GUARDS = '{%s}'.freeze
       DISCONNECT_ERRORS = /\A08S01/.freeze 
 
-      def initialize(opts)
-        super
-        case @opts[:db_type]
-        when 'mssql'
-          Sequel.ts_require 'adapters/odbc/mssql'
-          extend Sequel::ODBC::MSSQL::DatabaseMethods
-          @dataset_class = Sequel::ODBC::MSSQL::Dataset
-          set_mssql_unicode_strings
-        when 'progress'
-          Sequel.ts_require 'adapters/shared/progress'
-          extend Sequel::Progress::DatabaseMethods
-          extend_datasets(Sequel::Progress::DatasetMethods)
-        when 'db2'
-          Sequel.ts_require 'adapters/odbc/db2'
-          extend Sequel::ODBC::DB2::DatabaseMethods
-          @dataset_class = Sequel::ODBC::DB2::Dataset
-        end
-      end
-
       def connect(server)
         opts = server_opts(server)
         if opts.include? :driver
@@ -48,7 +29,11 @@ module Sequel
         conn
       end      
 
-      def execute(sql, opts={})
+      def disconnect_connection(c)
+        c.disconnect
+      end
+
+      def execute(sql, opts=OPTS)
         synchronize(opts[:server]) do |conn|
           begin
             r = log_yield(sql){conn.run(sql)}
@@ -62,7 +47,7 @@ module Sequel
         end
       end
       
-      def execute_dui(sql, opts={})
+      def execute_dui(sql, opts=OPTS)
         synchronize(opts[:server]) do |conn|
           begin
             log_yield(sql){conn.do(sql)}
@@ -71,10 +56,27 @@ module Sequel
           end
         end
       end
-      alias do execute_dui
 
       private
       
+      def adapter_initialize
+        case @opts[:db_type]
+        when 'mssql'
+          Sequel.require 'adapters/odbc/mssql'
+          extend Sequel::ODBC::MSSQL::DatabaseMethods
+          self.dataset_class = Sequel::ODBC::MSSQL::Dataset
+          set_mssql_unicode_strings
+        when 'progress'
+          Sequel.require 'adapters/shared/progress'
+          extend Sequel::Progress::DatabaseMethods
+          extend_datasets(Sequel::Progress::DatasetMethods)
+        when 'db2'
+          Sequel.require 'adapters/shared/db2'
+          extend ::Sequel::DB2::DatabaseMethods
+          extend_datasets ::Sequel::DB2::DatasetMethods
+        end
+      end
+
       def connection_execute_method
         :do
       end
@@ -83,10 +85,6 @@ module Sequel
         [::ODBC::Error]
       end
 
-      def disconnect_connection(c)
-        c.disconnect
-      end
-
       def disconnect_error?(e, opts)
         super || (e.is_a?(::ODBC::Error) && DISCONNECT_ERRORS.match(e.message))
       end
@@ -105,16 +103,11 @@ module Sequel
           i = -1
           cols = s.columns(true).map{|c| [output_identifier(c.name), i+=1]}
           columns = cols.map{|c| c.at(0)}
-          if opts[:offset] && offset_returns_row_number_column?
-            rn = row_number_column
-            columns.delete(rn)
-          end
           @columns = columns
           if rows = s.fetch_all
             rows.each do |row|
               hash = {}
               cols.each{|n,i| hash[n] = convert_odbc_value(row[i])}
-              hash.delete(rn) if rn
               yield hash
             end
           end
