@@ -13,18 +13,14 @@ module Sequel
       # ORA-03114: not connected to ORACLE
       CONNECTION_ERROR_CODES = [ 28, 1012, 3113, 3114 ]      
       
-      ORACLE_TYPES = {:blob=>lambda{|b| Sequel::SQL::Blob.new(b.read)}}
+      ORACLE_TYPES = {
+        :blob=>lambda{|b| Sequel::SQL::Blob.new(b.read)},
+        :clob=>lambda{|b| b.read}
+      }
 
       # Hash of conversion procs for this database.
       attr_reader :conversion_procs
 
-      def initialize(opts={})
-        super
-        @autosequence = opts[:autosequence]
-        @primary_key_sequences = {}
-        @conversion_procs = ORACLE_TYPES.dup
-      end
-
       def connect(server)
         opts = server_opts(server)
         if opts[:database]
@@ -34,7 +30,9 @@ module Sequel
           dbname = opts[:host]
         end
         conn = OCI8.new(opts[:user], opts[:password], dbname, opts[:privilege])
-        conn.prefetch_rows = typecast_value_integer(opts[:prefetch_rows]) if opts[:prefetch_rows]
+        if prefetch_rows = opts.fetch(:prefetch_rows, 100)
+          conn.prefetch_rows = typecast_value_integer(prefetch_rows)
+        end
         conn.autocommit = true
         conn.non_blocking = true
         
@@ -57,18 +55,23 @@ module Sequel
         conn
       end
 
-      def execute(sql, opts={}, &block)
+      def disconnect_connection(c)
+        c.logoff
+      rescue OCIInvalidHandle
+        nil
+      end
+
+      def execute(sql, opts=OPTS, &block)
         _execute(nil, sql, opts, &block)
       end
-      alias do execute
 
-      def execute_insert(sql, opts={})
+      def execute_insert(sql, opts=OPTS)
         _execute(:insert, sql, opts)
       end
 
       private
 
-      def _execute(type, sql, opts={}, &block)
+      def _execute(type, sql, opts=OPTS, &block)
         synchronize(opts[:server]) do |conn|
           begin
             return execute_prepared_statement(conn, type, sql, opts, &block) if sql.is_a?(Symbol)
@@ -98,6 +101,12 @@ module Sequel
         end
       end
 
+      def adapter_initialize
+        @autosequence = @opts[:autosequence]
+        @primary_key_sequences = {}
+        @conversion_procs = ORACLE_TYPES.dup
+      end
+
       PS_TYPES = {'string'.freeze=>String, 'integer'.freeze=>Integer, 'float'.freeze=>Float,
         'decimal'.freeze=>Float, 'date'.freeze=>Time, 'datetime'.freeze=>Time,
         'time'.freeze=>Time, 'boolean'.freeze=>String, 'blob'.freeze=>OCI8::BLOB}
@@ -125,6 +134,31 @@ module Sequel
         end
       end
 
+      def connection_execute_method
+        :exec
+      end
+
+      def database_error_classes
+        [OCIException, RuntimeError]
+      end
+
+      def database_specific_error_class(exception, opts)
+        case exception.code
+        when 1400, 1407
+          NotNullConstraintViolation
+        when 1
+          UniqueConstraintViolation
+        when 2291, 2292
+          ForeignKeyConstraintViolation
+        when 2290
+          CheckConstraintViolation
+        when 8177
+          SerializationFailure
+        else
+          super
+        end
+      end
+
       def execute_prepared_statement(conn, type, name, opts)
         ps = prepared_statement(name)
         sql = ps.prepared_sql
@@ -176,20 +210,15 @@ module Sequel
         end
       end
 
-      def begin_transaction(conn, opts={})
+      def begin_transaction(conn, opts=OPTS)
         log_yield(TRANSACTION_BEGIN){conn.autocommit = false}
+        set_transaction_isolation(conn, opts)
       end
       
-      def commit_transaction(conn, opts={})
+      def commit_transaction(conn, opts=OPTS)
         log_yield(TRANSACTION_COMMIT){conn.commit}
       end
 
-      def disconnect_connection(c)
-        c.logoff
-      rescue OCIInvalidHandle
-        nil
-      end
-
       def disconnect_error?(e, opts)
         super || (e.is_a?(::OCIError) && CONNECTION_ERROR_CODES.include?(e.code))
       end
@@ -218,11 +247,11 @@ module Sequel
         super
       end
       
-      def rollback_transaction(conn, opts={})
+      def rollback_transaction(conn, opts=OPTS)
         log_yield(TRANSACTION_ROLLBACK){conn.rollback}
       end
 
-      def schema_parse_table(table, opts={})
+      def schema_parse_table(table, opts=OPTS)
         schema, table = schema_and_table(table)
         schema ||= opts[:schema]
         schema_and_table = if ds = opts[:dataset]
@@ -326,17 +355,17 @@ module Sequel
         
         # Run execute_select on the database with the given SQL and the stored
         # bind arguments.
-        def execute(sql, opts={}, &block)
+        def execute(sql, opts=OPTS, &block)
           super(prepared_sql, {:arguments=>bind_arguments}.merge(opts), &block)
         end
         
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_dui(sql, opts={}, &block)
+        def execute_dui(sql, opts=OPTS, &block)
           super(prepared_sql, {:arguments=>bind_arguments}.merge(opts), &block)
         end
         
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_insert(sql, opts={}, &block)
+        def execute_insert(sql, opts=OPTS, &block)
           super(prepared_sql, {:arguments=>bind_arguments}.merge(opts), &block)
         end
       end
@@ -348,17 +377,17 @@ module Sequel
           
         # Execute the stored prepared statement name and the stored bind
         # arguments instead of the SQL given.
-        def execute(sql, opts={}, &block)
+        def execute(sql, opts=OPTS, &block)
           super(prepared_statement_name, opts, &block)
         end
          
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_dui(sql, opts={}, &block)
+        def execute_dui(sql, opts=OPTS, &block)
           super(prepared_statement_name, opts, &block)
         end
           
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_insert(sql, opts={}, &block)
+        def execute_insert(sql, opts=OPTS, &block)
           super(prepared_statement_name, opts, &block)
         end
       end
@@ -372,18 +401,14 @@ module Sequel
       
       def fetch_rows(sql)
         execute(sql) do |cursor|
-          offset = @opts[:offset]
-          rn = row_number_column
           cps = db.conversion_procs
           cols = columns = cursor.get_col_names.map{|c| output_identifier(c)}
           metadata = cursor.column_metadata
           cm = cols.zip(metadata).map{|c, m| [c, cps[m.data_type]]}
-          columns = cols.reject{|x| x == rn} if offset
           @columns = columns
           while r = cursor.fetch
             row = {}
             r.zip(cm).each{|v, (c, cp)| row[c] = ((v && cp) ? cp.call(v) : v)}
-            row.delete(rn) if offset
             yield row
           end
         end
