@@ -3,12 +3,14 @@ Sequel.require 'adapters/utils/emulate_offset_with_row_number'
 module Sequel
   module Oracle
     module DatabaseMethods
+      extend Sequel::Database::ResetIdentifierMangling
+
       TEMPORARY = 'GLOBAL TEMPORARY '.freeze
       AUTOINCREMENT = ''.freeze
 
       attr_accessor :autosequence
 
-      def create_sequence(name, opts={})
+      def create_sequence(name, opts=OPTS)
         self << create_sequence_sql(name, opts)
       end
 
@@ -34,12 +36,12 @@ module Sequel
         false
       end
 
-      def tables(opts={})
+      def tables(opts=OPTS)
         m = output_identifier_meth
         metadata_dataset.from(:tab).server(opts[:server]).select(:tname).filter(:tabtype => 'TABLE').map{|r| m.call(r[:tname])}
       end
 
-      def views(opts={}) 
+      def views(opts=OPTS) 
         m = output_identifier_meth
         metadata_dataset.from(:tab).server(opts[:server]).select(:tname).filter(:tabtype => 'VIEW').map{|r| m.call(r[:tname])}
       end 
@@ -49,6 +51,16 @@ module Sequel
         metadata_dataset.from(:tab).filter(:tname =>m.call(name), :tabtype => 'VIEW').count > 0 
       end 
 
+      # Oracle supports deferrable constraints.
+      def supports_deferrable_constraints?
+        true
+      end
+
+      # DB2 supports transaction isolation levels.
+      def supports_transaction_isolation_levels?
+        true
+      end
+
       private
 
       # Handle Oracle specific ALTER TABLE SQL
@@ -84,7 +96,7 @@ module Sequel
         AUTOINCREMENT
       end
 
-      def create_sequence_sql(name, opts={})
+      def create_sequence_sql(name, opts=OPTS)
         "CREATE SEQUENCE #{quote_identifier(name)} start with #{opts [:start_with]||1} increment by #{opts[:increment_by]||1} nomaxvalue"
       end
 
@@ -94,7 +106,7 @@ module Sequel
         create_statements.each{|sql| execute_ddl(sql)}
       end
 
-      def create_table_sql_list(name, generator, options={})
+      def create_table_sql_list(name, generator, options=OPTS)
         statements = [create_table_sql(name, generator, options)]
         drop_seq_statement = nil
         generator.columns.each do |c|
@@ -120,7 +132,7 @@ module Sequel
         [drop_seq_statement, statements]
       end
 
-      def create_trigger_sql(table, name, definition, opts={})
+      def create_trigger_sql(table, name, definition, opts=OPTS)
         events = opts[:events] ? Array(opts[:events]) : [:insert, :update, :delete]
         sql = <<-end_sql
           CREATE#{' OR REPLACE' if opts[:replace]} TRIGGER #{quote_identifier(name)}
@@ -131,6 +143,17 @@ module Sequel
         sql
       end
 
+      DATABASE_ERROR_REGEXPS = {
+        /unique constraint .+ violated/ => UniqueConstraintViolation,
+        /integrity constraint .+ violated/ => ForeignKeyConstraintViolation,
+        /check constraint .+ violated/ => CheckConstraintViolation,
+        /cannot insert NULL into|cannot update .+ to NULL/ => NotNullConstraintViolation,
+        /can't serialize access for this transaction/ => SerializationFailure,
+      }.freeze
+      def database_error_regexps
+        DATABASE_ERROR_REGEXPS
+      end
+
       def default_sequence_name(table, column)
         "seq_#{table}_#{column}"
       end
@@ -144,6 +167,16 @@ module Sequel
         super
       end
       
+      TRANSACTION_ISOLATION_LEVELS = {:uncommitted=>'READ COMMITTED'.freeze,
+        :committed=>'READ COMMITTED'.freeze,
+        :repeatable=>'SERIALIZABLE'.freeze,
+        :serializable=>'SERIALIZABLE'.freeze}
+      # Oracle doesn't support READ UNCOMMITTED OR REPEATABLE READ transaction
+      # isolation levels, so upgrade to the next highest level in those cases.
+      def set_transaction_isolation_sql(level)
+        "SET TRANSACTION ISOLATION LEVEL #{TRANSACTION_ISOLATION_LEVELS[level]}"
+      end
+    
       def sequence_for_table(table)
         return nil unless autosequence
         @primary_key_sequences.fetch(table) do |key|
@@ -155,6 +188,11 @@ module Sequel
         end
       end
 
+      # Oracle supports CREATE OR REPLACE VIEW.
+      def supports_create_or_replace_view?
+        true
+      end
+
       # Oracle's integer/:number type handles larger values than
       # most other databases's bigint types, so it should be
       # safe to use for Bignum.
@@ -178,6 +216,11 @@ module Sequel
       def temporary_table_sql
         TEMPORARY
       end
+
+      # Oracle uses clob for text types.
+      def uses_clob_for_text?
+        true
+      end
     end
 
     module DatasetMethods
@@ -192,12 +235,6 @@ module Sequel
       FROM = Dataset::FROM
       BITCOMP_OPEN = "((0 - ".freeze
       BITCOMP_CLOSE = ") - 1)".freeze
-      ILIKE_0 = "(UPPER(".freeze
-      ILIKE_1 = ") ".freeze
-      ILIKE_2 = ' UPPER('.freeze
-      ILIKE_3 = "))".freeze
-      LIKE = 'LIKE'.freeze
-      NOT_LIKE = 'NOT LIKE'.freeze
       TIMESTAMP_FORMAT = "TIMESTAMP '%Y-%m-%d %H:%M:%S%N %z'".freeze
       TIMESTAMP_OFFSET_FORMAT = "%+03i:%02i".freeze
       BOOL_FALSE = "'N'".freeze
@@ -205,33 +242,34 @@ module Sequel
       HSTAR = "H*".freeze
       DUAL = ['DUAL'.freeze].freeze
 
-      # Oracle needs to emulate bitwise operators and ILIKE/NOT ILIKE operators.
       def complex_expression_sql_append(sql, op, args)
         case op
         when :&
           sql << complex_expression_arg_pairs(args){|a, b| "CAST(BITAND(#{literal(a)}, #{literal(b)}) AS INTEGER)"}
         when :|
-          sql << complex_expression_arg_pairs(args){|a, b| "(#{literal(a)} - #{complex_expression_sql(:&, [a, b])} + #{literal(b)})"}
+          sql << complex_expression_arg_pairs(args) do |a, b|
+            s1 = ''
+            complex_expression_sql_append(s1, :&, [a, b])
+            "(#{literal(a)} - #{s1} + #{literal(b)})"
+          end
         when :^
-          sql << complex_expression_arg_pairs(args){|*x| "(#{complex_expression_sql(:|, x)} - #{complex_expression_sql(:&, x)})"}
+          sql << complex_expression_arg_pairs(args) do |*x|
+            s1 = ''
+            s2 = ''
+            complex_expression_sql_append(s1, :|, x)
+            complex_expression_sql_append(s2, :&, x)
+            "(#{s1} - #{s2})"
+          end
         when :'B~'
           sql << BITCOMP_OPEN
           literal_append(sql, args.at(0))
           sql << BITCOMP_CLOSE
         when :<<
-          sql << complex_expression_arg_pairs(args){|a, b| "(#{literal(a)} * power(2, #{literal b}))"}
+          sql << complex_expression_arg_pairs(args){|a, b| "(#{literal(a)} * power(2, #{literal(b)}))"}
         when :>>
-          sql << complex_expression_arg_pairs(args){|a, b| "(#{literal(a)} / power(2, #{literal b}))"}
+          sql << complex_expression_arg_pairs(args){|a, b| "(#{literal(a)} / power(2, #{literal(b)}))"}
         when :%
           sql << complex_expression_arg_pairs(args){|a, b| "MOD(#{literal(a)}, #{literal(b)})"}
-        when :ILIKE, :'NOT ILIKE'
-          sql << ILIKE_0
-          literal_append(sql, args.at(0))
-          sql << ILIKE_1
-          sql << (op == :ILIKE ? LIKE : NOT_LIKE)
-          sql<< ILIKE_2
-          literal_append(sql, args.at(1))
-          sql << ILIKE_3
         else
           super
         end
@@ -248,9 +286,22 @@ module Sequel
         end
       end
 
+      # Oracle treats empty strings like NULL values, and doesn't support
+      # char_length, so make char_length use length with a nonempty string.
+      # Unfortunately, as Oracle treats the empty string as NULL, there is
+      # no way to get trim to return an empty string instead of nil if
+      # the string only contains spaces.
+      def emulated_function_sql_append(sql, f)
+        case f.f
+        when :char_length
+          literal_append(sql, Sequel::SQL::Function.new(:length, Sequel.join([f.args.first, 'x'])) - 1)
+        else
+          super
+        end
+      end
+      
       # Oracle uses MINUS instead of EXCEPT, and doesn't support EXCEPT ALL
-      def except(dataset, opts={})
-        opts = {:all=>opts} unless opts.is_a?(Hash)
+      def except(dataset, opts=OPTS)
         raise(Sequel::Error, "EXCEPT ALL not supported") if opts[:all]
         compound_clone(:minus, dataset, opts)
       end
@@ -349,7 +400,7 @@ module Sequel
 
       # If this dataset is associated with a sequence, return the most recently
       # inserted sequence value.
-      def execute_insert(sql, opts={})
+      def execute_insert(sql, opts=OPTS)
         f = @opts[:from]
         super(sql, {:table=>(f.first if f), :sequence=>@opts[:sequence]}.merge(opts))
       end
