@@ -17,20 +17,15 @@
 #
 #   Sequel.sqlite('blog.db'){|db| puts db[:users].count} 
 #
-# You can set the +SEQUEL_NO_CORE_EXTENSIONS+ constant or environment variable to have
-# Sequel not extend the core classes.
-#
 # For a more expanded introduction, see the {README}[link:files/README_rdoc.html].
 # For a quicker introduction, see the {cheat sheet}[link:files/doc/cheat_sheet_rdoc.html].
 module Sequel
   @convert_two_digit_years = true
   @datetime_class = Time
-  @virtual_row_instance_eval = true
-  @require_thread = nil
-  
-  # Mutex used to protect file loading/requireing
-  @require_mutex = Mutex.new
-  
+
+  # Whether Sequel is being run in single threaded mode
+  @single_threaded = false
+
   class << self
     # Sequel converts two digit years in <tt>Date</tt>s and <tt>DateTime</tt>s by default,
     # so 01/02/03 is interpreted at January 2nd, 2003, and 12/13/99 is interpreted
@@ -53,28 +48,6 @@ module Sequel
     # they often implement them differently (e.g. + using seconds on +Time+ and
     # days on +DateTime+).
     attr_accessor :datetime_class
-
-    # For backwards compatibility, has no effect.
-    attr_accessor :virtual_row_instance_eval
-    
-    # Alias to the standard version of require
-    alias k_require require
-
-    private
-
-    # Make thread safe requiring reentrant to prevent deadlocks.
-    def check_requiring_thread
-      t = Thread.current
-      return(yield) if @require_thread == t
-      @require_mutex.synchronize do
-        begin
-          @require_thread = t 
-          yield
-        ensure
-          @require_thread = nil
-        end
-      end
-    end
   end
 
   # Returns true if the passed object could be a specifier of conditions, false otherwise.
@@ -91,12 +64,15 @@ module Sequel
     when Hash
       true
     when Array
-      !obj.empty? && !obj.is_a?(SQL::ValueList) && obj.all?{|i| (Array === i) && (i.length == 2)}
+      !obj.empty? && !obj.is_a?(SQL::ValueList) && obj.all?{|i| i.is_a?(Array) && (i.length == 2)}
     else
       false
     end
   end
 
+  # Frozen hash used as the default options hash for most options.
+  OPTS = {}.freeze
+
   # Creates a new database object based on the supplied connection string
   # and optional arguments.  The specified scheme determines the database
   # class used, and the rest of the string specifies the connection options.
@@ -118,7 +94,13 @@ module Sequel
   def self.connect(*args, &block)
     Database.connect(*args, &block)
   end
-  
+
+  # Assume the core extensions are not loaded by default, if the core_extensions
+  # extension is loaded, this will be overridden.
+  def self.core_extensions?
+    false
+  end
+
   # Convert the +exception+ to the given class.  The given class should be
   # <tt>Sequel::Error</tt> or a subclass.  Returns an instance of +klass+ with
   # the message and backtrace of +exception+.
@@ -141,7 +123,7 @@ module Sequel
   #   Sequel.extension(:schema_dumper)
   #   Sequel.extension(:pagination, :query)
   def self.extension(*extensions)
-    extensions.each{|e| tsk_require "sequel/extensions/#{e}"}
+    extensions.each{|e| Kernel.require "sequel/extensions/#{e}"}
   end
   
   # Set the method to call on identifiers going into the database.  This affects
@@ -158,7 +140,7 @@ module Sequel
   def self.identifier_input_method=(value)
     Database.identifier_input_method = value
   end
-  
+
   # Set the method to call on identifiers coming out of the database.  This affects
   # the literalization of identifiers by calling this method on them when they are
   # retrieved from the database.  Sequel downcases identifiers retrieved for most
@@ -174,7 +156,25 @@ module Sequel
   def self.identifier_output_method=(value)
     Database.identifier_output_method = value
   end
-  
+
+  # The exception classed raised if there is an error parsing JSON.
+  # This can be overridden to use an alternative json implementation.
+  def self.json_parser_error_class
+    JSON::ParserError
+  end
+
+  # Convert given object to json and return the result.
+  # This can be overridden to use an alternative json implementation.
+  def self.object_to_json(obj, *args)
+    obj.to_json(*args)
+  end
+
+  # Parse the string as JSON and return the result.
+  # This can be overridden to use an alternative json implementation.
+  def self.parse_json(json)
+    JSON.parse(json, :create_additions=>false)
+  end
+
   # Set whether to quote identifiers for all databases by default. By default,
   # Sequel quotes identifiers in all SQL strings, so to turn that off:
   #
@@ -182,7 +182,20 @@ module Sequel
   def self.quote_identifiers=(value)
     Database.quote_identifiers = value
   end
-  
+
+  # Convert each item in the array to the correct type, handling multi-dimensional
+  # arrays.  For each element in the array or subarrays, call the converter,
+  # unless the value is nil.
+  def self.recursive_map(array, converter)
+    array.map do |i|
+      if i.is_a?(Array)
+        recursive_map(i, converter)
+      elsif i
+        converter.call(i)
+      end
+    end
+  end
+
   # Require all given +files+ which should be in the same or a subdirectory of
   # this file.  If a +subdir+ is given, assume all +files+ are in that subdir.
   # This is used to ensure that the files loaded are from the same version of
@@ -190,17 +203,41 @@ module Sequel
   def self.require(files, subdir=nil)
     Array(files).each{|f| super("#{File.dirname(__FILE__).untaint}/#{"#{subdir}/" if subdir}#{f}")}
   end
-  
-  # Set whether to set the single threaded mode for all databases by default. By default,
+
+  # Set whether Sequel is being used in single threaded mode. By default,
   # Sequel uses a thread-safe connection pool, which isn't as fast as the
-  # single threaded connection pool.  If your program will only have one thread,
-  # and speed is a priority, you may want to set this to true:
+  # single threaded connection pool, and also has some additional thread
+  # safety checks.  If your program will only have one thread,
+  # and speed is a priority, you should set this to true:
   #
   #   Sequel.single_threaded = true
   def self.single_threaded=(value)
+    @single_threaded = value
     Database.single_threaded = value
   end
 
+  COLUMN_REF_RE1 = /\A((?:(?!__).)+)__((?:(?!___).)+)___(.+)\z/.freeze
+  COLUMN_REF_RE2 = /\A((?:(?!___).)+)___(.+)\z/.freeze
+  COLUMN_REF_RE3 = /\A((?:(?!__).)+)__(.+)\z/.freeze
+
+  # Splits the symbol into three parts.  Each part will
+  # either be a string or nil.
+  #
+  # For columns, these parts are the table, column, and alias.
+  # For tables, these parts are the schema, table, and alias.
+  def self.split_symbol(sym)
+    case s = sym.to_s
+    when COLUMN_REF_RE1
+      [$1, $2, $3]
+    when COLUMN_REF_RE2
+      [nil, $1, $2]
+    when COLUMN_REF_RE3
+      [$1, $2, nil]
+    else
+      [nil, s, nil]
+    end
+  end
+
   # Converts the given +string+ into a +Date+ object.
   #
   #   Sequel.string_to_date('2010-09-10') # Date.civil(2010, 09, 10)
@@ -240,6 +277,26 @@ module Sequel
     end
   end
 
+  if defined?(RUBY_ENGINE) && RUBY_ENGINE != 'ruby'
+  # :nocov:
+    # Mutex used to protect mutable data structures
+    @data_mutex = Mutex.new
+
+    # Unless in single threaded mode, protects access to any mutable
+    # global data structure in Sequel.
+    # Uses a non-reentrant mutex, so calling code should be careful.
+    def self.synchronize(&block)
+      @single_threaded ? yield : @data_mutex.synchronize(&block)
+    end
+  # :nocov:
+  else
+    # Yield directly to the block.  You don't need to synchronize
+    # access on MRI because the GVL makes certain methods atomic.
+    def self.synchronize
+      yield
+    end
+  end
+
   # Uses a transaction on all given databases with the given options. This:
   #
   #   Sequel.transaction([DB1, DB2, DB3]){...}
@@ -263,7 +320,7 @@ module Sequel
   # to uncommit the changes on DB3.  For that kind of support, you need to
   # have two-phase commit/prepared transactions (which Sequel supports on
   # some databases).
-  def self.transaction(dbs, opts={}, &block)
+  def self.transaction(dbs, opts=OPTS, &block)
     unless opts[:rollback]
       rescue_rollback = true
       opts = opts.merge(:rollback=>:reraise)
@@ -272,7 +329,7 @@ module Sequel
     if rescue_rollback
       begin
         pr.call
-      rescue Sequel::Rollback => e
+      rescue Sequel::Rollback
         nil
       end
     else
@@ -280,51 +337,43 @@ module Sequel
     end
   end
 
-  # Same as Sequel.require, but wrapped in a mutex in order to be thread safe.
-  def self.ts_require(*args)
-    check_requiring_thread{require(*args)}
-  end
-  
-  # Same as Kernel.require, but wrapped in a mutex in order to be thread safe.
-  def self.tsk_require(*args)
-    check_requiring_thread{k_require(*args)}
-  end
-
   # If the supplied block takes a single argument,
-  # yield a new <tt>SQL::VirtualRow</tt> instance to the block
-  # argument.  Otherwise, evaluate the block in the context of a new
+  # yield an <tt>SQL::VirtualRow</tt> instance to the block
+  # argument.  Otherwise, evaluate the block in the context of a
   # <tt>SQL::VirtualRow</tt> instance.
   #
   #   Sequel.virtual_row{a} # Sequel::SQL::Identifier.new(:a)
   #   Sequel.virtual_row{|o| o.a{}} # Sequel::SQL::Function.new(:a)
   def self.virtual_row(&block)
-    vr = SQL::VirtualRow.new
+    vr = VIRTUAL_ROW
     case block.arity
     when -1, 0
-      vr.instance_eval(&block)
+      vr.instance_exec(&block)
     else
       block.call(vr)
     end  
   end
-  
+
   ### Private Class Methods ###
 
   # Helper method that the database adapter class methods that are added to Sequel via
   # metaprogramming use to parse arguments.
-  def self.adapter_method(adapter, *args, &block) # :nodoc:
-    raise(::Sequel::Error, "Wrong number of arguments, 0-2 arguments valid") if args.length > 2
-    opts = {:adapter=>adapter.to_sym}
-    opts[:database] = args.shift if args.length >= 1 && !(args[0].is_a?(Hash))
-    if Hash === (arg = args[0])
-      opts.merge!(arg)
-    elsif !arg.nil?
+  def self.adapter_method(adapter, *args, &block)
+    options = args.last.is_a?(Hash) ? args.pop : {}
+    opts = {:adapter => adapter.to_sym}
+    opts[:database] = args.shift if args.first.is_a?(String)
+    if args.any?
       raise ::Sequel::Error, "Wrong format of arguments, either use (), (String), (Hash), or (String, Hash)"
     end
-    connect(opts, &block)
+
+    connect(opts.merge(options), &block)
   end
 
   # Method that adds a database adapter class method to Sequel that calls
   # Sequel.adapter_method.
+  #
+  # Do not call this method with untrusted input, as that can result in
+  # arbitrary code execution.
   def self.def_adapter_method(*adapters) # :nodoc:
     adapters.each do |adapter|
       instance_eval("def #{adapter}(*args, &block); adapter_method('#{adapter}', *args, &block) end", __FILE__, __LINE__)
@@ -332,9 +381,8 @@ module Sequel
   end
 
   private_class_method :adapter_method, :def_adapter_method
-  
-  require(%w"metaprogramming sql connection_pool exceptions dataset database timezones ast_transformer version")
-  require('core_sql') if !defined?(::SEQUEL_NO_CORE_EXTENSIONS) && !ENV.has_key?('SEQUEL_NO_CORE_EXTENSIONS')
+
+  require(%w"deprecated sql connection_pool exceptions dataset database timezones ast_transformer version")
 
   # Add the database adapter class methods to Sequel via metaprogramming
   def_adapter_method(*Database::ADAPTERS)
