@@ -10,10 +10,8 @@ module Sequel
     def quote_identifiers?
       if defined?(@quote_identifiers)
         @quote_identifiers
-      elsif db.respond_to?(:quote_identifiers?)
-        @quote_identifiers = db.quote_identifiers?
       else
-        @quote_identifiers = false
+        @quote_identifiers = db.quote_identifiers?
       end
     end
     
@@ -61,6 +59,16 @@ module Sequel
       false
     end
 
+    # Whether the dataset supports CUBE with GROUP BY.
+    def supports_group_cube?
+      false
+    end
+
+    # Whether the dataset supports ROLLUP with GROUP BY.
+    def supports_group_rollup?
+      false
+    end
+
     # Whether this dataset supports the +insert_select+ method for returning all columns values
     # directly from an insert query.
     def supports_insert_select?
@@ -104,6 +112,11 @@ module Sequel
       supports_distinct_on?
     end
     
+    # Whether the dataset supports pattern matching by regular expressions.
+    def supports_regexp?
+      false
+    end
+
     # Whether the RETURNING clause is supported for the given type of query.
     # +type+ can be :insert, :update, or :delete.
     def supports_returning?(type)
@@ -144,16 +157,15 @@ module Sequel
       true
     end
 
-    # Whether using an offset returns an extra row number column that should be
-    # eliminated, false by default.
-    def offset_returns_row_number_column?
-      false
-    end
-
     # Whether the RETURNING clause is used for the given dataset.
     # +type+ can be :insert, :update, or :delete.
     def uses_returning?(type)
       opts[:returning] && !@opts[:sql] && supports_returning?(type)
     end
+    
+    # Whether the dataset uses WITH ROLLUP/CUBE instead of ROLLUP()/CUBE().
+    def uses_with_rollup?
+      false
+    end
   end
 end
