@@ -43,10 +43,10 @@ describe "A new Database" do
     Sequel::Database.new(:max_connections => 10).pool.max_size.should == 10
   end
   
-  specify "should pass the supplied block to the connection pool" do
+  specify "should have the connection pool use the connect method to get connections" do
     cc = nil
     d = Sequel::Database.new
-    d.meta_def(:connect){|c| 1234}
+    meta_def(d, :connect){|c| 1234}
     d.synchronize {|c| cc = c}
     cc.should == 1234
   end
@@ -81,7 +81,7 @@ describe "A new Database" do
 
   specify "should respect the :identifier_input_method option" do
     Sequel.identifier_input_method = nil
-    Sequel::Database.identifier_input_method.should == ""
+    Sequel::Database.identifier_input_method.should == false
     db = Sequel::Database.new(:identifier_input_method=>nil)
     db.identifier_input_method.should be_nil
     db.identifier_input_method = :downcase
@@ -104,7 +104,7 @@ describe "A new Database" do
   
   specify "should respect the :identifier_output_method option" do
     Sequel.identifier_output_method = nil
-    Sequel::Database.identifier_output_method.should == ""
+    Sequel::Database.identifier_output_method.should == false
     db = Sequel::Database.new(:identifier_output_method=>nil)
     db.identifier_output_method.should be_nil
     db.identifier_output_method = :downcase
@@ -169,7 +169,7 @@ describe "A new Database" do
   
   specify "should respect the identifier_input_method_default method" do
     class Sequel::Database
-      @@identifier_input_method = nil
+      @identifier_input_method = nil
     end
     x = Class.new(Sequel::Database){def identifier_input_method_default; :downcase end}
     x.new({}).identifier_input_method.should == :downcase
@@ -179,7 +179,7 @@ describe "A new Database" do
   
   specify "should respect the identifier_output_method_default method if Sequel.identifier_output_method is not called" do
     class Sequel::Database
-      @@identifier_output_method = nil
+      @identifier_output_method = nil
     end
     x = Class.new(Sequel::Database){def identifier_output_method_default; :upcase end}
     x.new({}).identifier_output_method.should == :upcase
@@ -204,6 +204,11 @@ describe "A new Database" do
   specify "should populate :adapter option when using connection string" do
     Sequel.connect('mock:/').opts[:adapter].should == "mock"
   end
+
+  specify "should respect the :keep_reference option for not keeping a reference in Sequel::DATABASES" do
+    db = Sequel.connect('mock:///?keep_reference=f')
+    Sequel::DATABASES.should_not include(db)
+  end
 end
 
 describe "Database#disconnect" do
@@ -221,12 +226,6 @@ describe "Sequel.extension" do
   end
 end
 
-describe "Database#connect" do
-  specify "should raise Sequel::NotImplemented" do
-    proc {Sequel::Database.new.connect(:default)}.should raise_error(Sequel::NotImplemented)
-  end
-end
-
 describe "Database#log_info" do
   before do
     @o = Object.new
@@ -343,7 +342,7 @@ describe "Database#uri" do
 end
 
 describe "Database.adapter_scheme and #adapter_scheme" do
-  specify "should return the database schema" do
+  specify "should return the database scheme" do
     Sequel::Database.adapter_scheme.should be_nil
 
     @c = Class.new(Sequel::Database) do
@@ -411,9 +410,9 @@ describe "Database#dataset_class" do
   end
 
   specify "should have getter return the class to use to create datasets" do
-    @db.dataset_class.should == Sequel::Dataset
+    [@db.dataset_class, @db.dataset_class.superclass].should include(Sequel::Dataset)
     @db.dataset_class = @dsc
-    @db.dataset_class.should == @dsc
+    [@db.dataset_class, @db.dataset_class.superclass].should include(@dsc)
   end
 end
   
@@ -470,7 +469,7 @@ describe "Database#extend_datasets" do
     @db[:t].select(:a, :b).sql.should == 'SELECT a, b FROM t ORDER BY a, b'
   end
 
-  specify "should reapply settings if dataset_class is chagned" do
+  specify "should reapply settings if dataset_class is changed" do
     c = Class.new(Sequel::Dataset)
     @db.dataset_class = c
     @db.dataset_class.superclass.should == c
@@ -479,33 +478,23 @@ describe "Database#extend_datasets" do
   end
 end
   
-describe "Database#execute" do
-  specify "should raise Sequel::NotImplemented" do
-    proc {Sequel::Database.new.execute('blah blah')}.should raise_error(Sequel::NotImplemented)
+describe "Database#disconnect_connection" do
+  specify "should call close on the connection" do
+    o = Object.new
+    def o.close() @closed=true end
+    Sequel::Database.new.disconnect_connection(o)
+    o.instance_variable_get(:@closed).should be_true
   end
 end
 
-describe "Database#tables" do
-  specify "should raise Sequel::NotImplemented" do
-    proc {Sequel::Database.new.tables}.should raise_error(Sequel::NotImplemented)
-  end
-end
-
-describe "Database#views" do
-  specify "should raise Sequel::NotImplemented" do
-    proc {Sequel::Database.new.views}.should raise_error(Sequel::NotImplemented)
-  end
-end
-
-describe "Database#indexes" do
-  specify "should raise Sequel::NotImplemented" do
-    proc {Sequel::Database.new.indexes(:table)}.should raise_error(Sequel::NotImplemented)
-  end
-end
-
-describe "Database#foreign_key_list" do
-  specify "should raise Sequel::NotImplemented" do
-    proc {Sequel::Database.new.foreign_key_list(:table)}.should raise_error(Sequel::NotImplemented)
+describe "Database#valid_connection?" do
+  specify "should issue a query to validate the connection" do
+    db = Sequel.mock
+    db.synchronize{|c| db.valid_connection?(c)}.should be_true
+    db.synchronize do |c|
+      def c.execute(*) raise Sequel::DatabaseError, "error" end
+      db.valid_connection?(c)
+    end.should be_false
   end
 end
 
@@ -548,11 +537,11 @@ end
 describe "Database#synchronize" do
   before do
     @db = Sequel::Database.new(:max_connections => 1)
-    @db.meta_def(:connect){|c| 12345}
+    meta_def(@db, :connect){|c| 12345}
   end
   
   specify "should wrap the supplied block in pool.hold" do
-    q, q1, q2, q3 = Queue.new, Queue.new, Queue.new, Queue.new
+    q, q1, q2 = Queue.new, Queue.new, Queue.new
     c1, c2 = nil
     t1 = Thread.new{@db.synchronize{|c| c1 = c; q.push nil; q1.pop}; q.push nil}
     q.pop
@@ -573,7 +562,7 @@ describe "Database#test_connection" do
   before do
     @db = Sequel::Database.new
     pr = proc{@test = rand(100)}
-    @db.meta_def(:connect){|c| pr.call}
+    meta_def(@db, :connect){|c| pr.call}
   end
   
   specify "should attempt to get a connection" do
@@ -586,15 +575,16 @@ describe "Database#test_connection" do
   end
 
   specify "should raise an error if the attempting to connect raises an error" do
-    proc{Sequel::Database.new{raise Sequel::Error, 'blah'}.test_connection}.should raise_error(Sequel::Error)
+    def @db.connect(*) raise Sequel::Error end
+    proc{@db.test_connection}.should raise_error(Sequel::Error)
   end
 end
 
 describe "Database#table_exists?" do
-  specify "should try to select the first record from the table's dataset" do
+  specify "should test existence by selecting a row from the table's dataset" do
     db = Sequel.mock(:fetch=>[Sequel::Error, [], [{:a=>1}]])
     db.table_exists?(:a).should be_false
-    db.sqls.should == ["SELECT NULL FROM a LIMIT 1"]
+    db.sqls.should == ["SELECT NULL AS nil FROM a LIMIT 1"]
     db.table_exists?(:b).should be_true
     db.table_exists?(:c).should be_true
   end
@@ -607,7 +597,7 @@ shared_examples_for "Database#transaction" do
   end
   
   specify "should support transaction isolation levels" do
-    @db.meta_def(:supports_transaction_isolation_levels?){true}
+    meta_def(@db, :supports_transaction_isolation_levels?){true}
     [:uncommitted, :committed, :repeatable, :serializable].each do |l|
       @db.transaction(:isolation=>l){@db.run "DROP TABLE #{l}"}
     end
@@ -618,7 +608,7 @@ shared_examples_for "Database#transaction" do
   end
 
   specify "should allow specifying a default transaction isolation level" do
-    @db.meta_def(:supports_transaction_isolation_levels?){true}
+    meta_def(@db, :supports_transaction_isolation_levels?){true}
     [:uncommitted, :committed, :repeatable, :serializable].each do |l|
       @db.transaction_isolation_level = l
       @db.transaction{@db.run "DROP TABLE #{l}"}
@@ -629,15 +619,50 @@ shared_examples_for "Database#transaction" do
                        'BEGIN', 'SET TRANSACTION ISOLATION LEVEL SERIALIZABLE', 'DROP TABLE serializable', 'COMMIT']
   end
   
-  specify "should support :disconnect=>:retry option for automatically retrying on disconnect" do
+  specify "should support :retry_on option for automatically retrying transactions" do
     a = []
-    @db.transaction(:disconnect=>:retry){a << 1; raise Sequel::DatabaseDisconnectError if a.length < 2}
+    @db.transaction(:retry_on=>Sequel::DatabaseDisconnectError){a << 1; raise Sequel::DatabaseDisconnectError if a.length < 2}
     @db.sqls.should == ['BEGIN', 'ROLLBACK', 'BEGIN', 'COMMIT']
     a.should == [1, 1]
+
+    a = []
+    @db.transaction(:retry_on=>[Sequel::ConstraintViolation, Sequel::SerializationFailure]) do
+      a << 1
+      raise Sequel::SerializationFailure if a.length == 1
+      raise Sequel::ConstraintViolation if a.length == 2
+    end
+    @db.sqls.should == ['BEGIN', 'ROLLBACK', 'BEGIN', 'ROLLBACK', 'BEGIN', 'COMMIT']
+    a.should == [1, 1, 1]
+  end
+  
+  specify "should support :num_retries option for limiting the number of retry times" do
+    a = []
+    lambda do
+      @db.transaction(:num_retries=>1, :retry_on=>[Sequel::ConstraintViolation, Sequel::SerializationFailure]) do
+        a << 1
+        raise Sequel::SerializationFailure if a.length == 1
+        raise Sequel::ConstraintViolation if a.length == 2
+      end
+    end.should raise_error(Sequel::ConstraintViolation)
+    @db.sqls.should == ['BEGIN', 'ROLLBACK', 'BEGIN', 'ROLLBACK']
+    a.should == [1, 1]
   end
   
-  specify "should raise an error if attempting to use :disconnect=>:retry inside another transaction" do
-    proc{@db.transaction{@db.transaction(:disconnect=>:retry){}}}.should raise_error(Sequel::Error)
+  specify "should support :num_retries=>nil option to retry indefinitely" do
+    a = []
+    lambda do
+      @db.transaction(:num_retries=>nil, :retry_on=>[Sequel::ConstraintViolation]) do
+        a << 1
+        raise Sequel::SerializationFailure if a.length >= 100
+        raise Sequel::ConstraintViolation
+      end
+    end.should raise_error(Sequel::SerializationFailure)
+    @db.sqls.should == ['BEGIN', 'ROLLBACK'] * 100
+    a.should == [1] * 100
+  end
+  
+  specify "should raise an error if attempting to use :retry_on inside another transaction" do
+    proc{@db.transaction{@db.transaction(:retry_on=>Sequel::ConstraintViolation){}}}.should raise_error(Sequel::Error)
     @db.sqls.should == ['BEGIN', 'ROLLBACK']
   end
   
@@ -646,7 +671,6 @@ shared_examples_for "Database#transaction" do
       transaction do
         execute 'DROP TABLE test;'
         return
-        execute 'DROP TABLE test2;';
       end
     end
     @db.ret_commit
@@ -662,8 +686,8 @@ shared_examples_for "Database#transaction" do
   
   specify "should handle errors when sending BEGIN" do
     ec = Class.new(StandardError)
-    @db.meta_def(:database_error_classes){[ec]}
-    @db.meta_def(:log_connection_execute){|c, sql| sql =~ /BEGIN/ ? raise(ec, 'bad') : super(c, sql)}
+    meta_def(@db, :database_error_classes){[ec]}
+    meta_def(@db, :log_connection_execute){|c, sql| sql =~ /BEGIN/ ? raise(ec, 'bad') : super(c, sql)}
     begin
       @db.transaction{@db.execute 'DROP TABLE test;'}
     rescue Sequel::DatabaseError => e
@@ -675,8 +699,8 @@ shared_examples_for "Database#transaction" do
   
   specify "should handle errors when sending COMMIT" do
     ec = Class.new(StandardError)
-    @db.meta_def(:database_error_classes){[ec]}
-    @db.meta_def(:log_connection_execute){|c, sql| sql =~ /COMMIT/ ? raise(ec, 'bad') : super(c, sql)}
+    meta_def(@db, :database_error_classes){[ec]}
+    meta_def(@db, :log_connection_execute){|c, sql| sql =~ /COMMIT/ ? raise(ec, 'bad') : super(c, sql)}
     begin
       @db.transaction{@db.execute 'DROP TABLE test;'}
     rescue Sequel::DatabaseError => e
@@ -688,8 +712,8 @@ shared_examples_for "Database#transaction" do
   
   specify "should handle errors when sending ROLLBACK" do
     ec = Class.new(StandardError)
-    @db.meta_def(:database_error_classes){[ec]}
-    @db.meta_def(:log_connection_execute){|c, sql| sql =~ /ROLLBACK/ ? raise(ec, 'bad') : super(c, sql)}
+    meta_def(@db, :database_error_classes){[ec]}
+    meta_def(@db, :log_connection_execute){|c, sql| sql =~ /ROLLBACK/ ? raise(ec, 'bad') : super(c, sql)}
     begin
       @db.transaction{raise ArgumentError, 'asdf'}
     rescue Sequel::DatabaseError => e
@@ -746,17 +770,17 @@ shared_examples_for "Database#transaction" do
     @db.sqls.should == ['BEGIN', 'ROLLBACK']
     @db.transaction(:rollback=>:always){1}.should be_nil
     @db.sqls.should == ['BEGIN', 'ROLLBACK']
-    catch (:foo) do
+    catch(:foo) do
       @db.transaction(:rollback=>:always){throw :foo}
     end
     @db.sqls.should == ['BEGIN', 'ROLLBACK']
   end
   
   specify "should raise database errors when commiting a transaction as Sequel::DatabaseError" do
-    @db.meta_def(:commit_transaction){raise ArgumentError}
+    meta_def(@db, :commit_transaction){raise ArgumentError}
     lambda{@db.transaction{}}.should raise_error(ArgumentError)
 
-    @db.meta_def(:database_error_classes){[ArgumentError]}
+    meta_def(@db, :database_error_classes){[ArgumentError]}
     lambda{@db.transaction{}}.should raise_error(Sequel::DatabaseError)
   end
   
@@ -873,13 +897,13 @@ shared_examples_for "Database#transaction" do
   end
 
   specify "should raise an error if you attempt to use after_commit inside a prepared transaction" do
-    @db.meta_def(:supports_prepared_transactions?){true}
+    meta_def(@db, :supports_prepared_transactions?){true}
     proc{@db.transaction(:prepare=>'XYZ'){@db.after_commit{@db.execute('foo')}}}.should raise_error(Sequel::Error)
     @db.sqls.should == ['BEGIN', 'ROLLBACK']
   end
 
   specify "should raise an error if you attempt to use after_rollback inside a prepared transaction" do
-    @db.meta_def(:supports_prepared_transactions?){true}
+    meta_def(@db, :supports_prepared_transactions?){true}
     proc{@db.transaction(:prepare=>'XYZ'){@db.after_rollback{@db.execute('foo')}}}.should raise_error(Sequel::Error)
     @db.sqls.should == ['BEGIN', 'ROLLBACK']
   end
@@ -893,7 +917,7 @@ describe "Database#transaction with savepoint support" do
   it_should_behave_like "Database#transaction"
 
   specify "should support after_commit inside savepoints" do
-    @db.meta_def(:supports_savepoints?){true}
+    meta_def(@db, :supports_savepoints?){true}
     @db.transaction do
       @db.after_commit{@db.execute('foo')}
       @db.transaction(:savepoint=>true){@db.after_commit{@db.execute('bar')}}
@@ -903,7 +927,7 @@ describe "Database#transaction with savepoint support" do
   end
 
   specify "should support after_rollback inside savepoints" do
-    @db.meta_def(:supports_savepoints?){true}
+    meta_def(@db, :supports_savepoints?){true}
     @db.transaction do
       @db.after_rollback{@db.execute('foo')}
       @db.transaction(:savepoint=>true){@db.after_rollback{@db.execute('bar')}}
@@ -914,15 +938,15 @@ describe "Database#transaction with savepoint support" do
   end
 
   specify "should raise an error if you attempt to use after_commit inside a savepoint in a prepared transaction" do
-    @db.meta_def(:supports_savepoints?){true}
-    @db.meta_def(:supports_prepared_transactions?){true}
+    meta_def(@db, :supports_savepoints?){true}
+    meta_def(@db, :supports_prepared_transactions?){true}
     proc{@db.transaction(:prepare=>'XYZ'){@db.transaction(:savepoint=>true){@db.after_commit{@db.execute('foo')}}}}.should raise_error(Sequel::Error)
     @db.sqls.should == ['BEGIN', 'SAVEPOINT autopoint_1','ROLLBACK TO SAVEPOINT autopoint_1', 'ROLLBACK']
   end
 
   specify "should raise an error if you attempt to use after_rollback inside a savepoint in a prepared transaction" do
-    @db.meta_def(:supports_savepoints?){true}
-    @db.meta_def(:supports_prepared_transactions?){true}
+    meta_def(@db, :supports_savepoints?){true}
+    meta_def(@db, :supports_prepared_transactions?){true}
     proc{@db.transaction(:prepare=>'XYZ'){@db.transaction(:savepoint=>true){@db.after_rollback{@db.execute('foo')}}}}.should raise_error(Sequel::Error)
     @db.sqls.should == ['BEGIN', 'SAVEPOINT autopoint_1','ROLLBACK TO SAVEPOINT autopoint_1', 'ROLLBACK']
   end
@@ -931,7 +955,7 @@ end
 describe "Database#transaction without savepoint support" do
   before do
     @db = Sequel.mock(:servers=>{:test=>{}})
-    @db.meta_def(:supports_savepoints?){false}
+    meta_def(@db, :supports_savepoints?){false}
   end
 
   it_should_behave_like "Database#transaction"
@@ -989,7 +1013,7 @@ describe "Database#transaction with savepoints" do
     @db.sqls.should == ['BEGIN', 'SAVEPOINT autopoint_1', 'DROP TABLE test;', 'RELEASE SAVEPOINT autopoint_1', 'COMMIT']
   end
   
-  specify "should not use a savepoints if no transaction is in progress" do
+  specify "should not use a savepoint if no transaction is in progress" do
     @db.transaction(:savepoint=>true){@db.execute 'DROP TABLE test;'}
     @db.sqls.should == ['BEGIN', 'DROP TABLE test;', 'COMMIT']
   end
@@ -1004,7 +1028,6 @@ describe "Database#transaction with savepoints" do
       transaction do
         execute 'DROP TABLE test;'
         return
-        execute 'DROP TABLE test2;';
       end
     end
     @db.ret_commit
@@ -1017,7 +1040,6 @@ describe "Database#transaction with savepoints" do
         transaction(:savepoint=>true) do
           execute 'DROP TABLE test;'
           return
-          execute 'DROP TABLE test2;';
         end
       end
     end
@@ -1062,11 +1084,11 @@ describe "Database#transaction with savepoints" do
   end
   
   specify "should raise database errors when commiting a transaction as Sequel::DatabaseError" do
-    @db.meta_def(:commit_transaction){raise ArgumentError}
+    meta_def(@db, :commit_transaction){raise ArgumentError}
     lambda{@db.transaction{}}.should raise_error(ArgumentError)
     lambda{@db.transaction{@db.transaction(:savepoint=>true){}}}.should raise_error(ArgumentError)
 
-    @db.meta_def(:database_error_classes){[ArgumentError]}
+    meta_def(@db, :database_error_classes){[ArgumentError]}
     lambda{@db.transaction{}}.should raise_error(Sequel::DatabaseError)
     lambda{@db.transaction{@db.transaction(:savepoint=>true){}}}.should raise_error(Sequel::DatabaseError)
   end
@@ -1074,7 +1096,7 @@ end
 
 describe "A Database adapter with a scheme" do
   before do
-    @ccc = Class.new(Sequel::Database)
+    @ccc = Class.new(Sequel::Mock::Database)
     @ccc.send(:set_adapter_scheme, :ccc)
   end
 
@@ -1131,6 +1153,7 @@ describe "A Database adapter with a scheme" do
     Sequel.send(:def_adapter_method, :ccc)
     Sequel.ccc('db', :host=>'localhost', &p).should == returnValue
     @ccc::DISCONNECTS.should == [z, y, x]
+    class << Sequel; remove_method(:ccc) end
   end
 
   specify "should be accessible through Sequel.<adapter>" do
@@ -1138,6 +1161,7 @@ describe "A Database adapter with a scheme" do
 
     # invalid parameters
     proc {Sequel.ccc('abc', 'def')}.should raise_error(Sequel::Error)
+    proc {Sequel.ccc(1)}.should raise_error(Sequel::Error)
     
     c = Sequel.ccc('mydb')
     c.should be_a_kind_of(@ccc)
@@ -1154,6 +1178,7 @@ describe "A Database adapter with a scheme" do
     c = Sequel.ccc(:database => 'mydb', :host => 'localhost')
     c.should be_a_kind_of(@ccc)
     c.opts.values_at(:adapter, :database, :host, :adapter_class).should == [:ccc, 'mydb', 'localhost', @ccc]
+    class << Sequel; remove_method(:ccc) end
   end
   
   specify "should be accessible through Sequel.connect with options" do
@@ -1193,6 +1218,7 @@ describe "A Database adapter with a scheme" do
   end
 
   specify "should test the connection if test parameter is truthy" do
+    @ccc.send(:define_method, :connect){}
     proc{Sequel.connect 'ccc:///d%5bb%5d?test=t'}.should raise_error(Sequel::DatabaseConnectionError)
     proc{Sequel.connect 'ccc:///d%5bb%5d?test=1'}.should raise_error(Sequel::DatabaseConnectionError)
     proc{Sequel.connect 'ccc:///d%5bb%5d', :test=>true}.should raise_error(Sequel::DatabaseConnectionError)
@@ -1271,7 +1297,7 @@ describe "A single threaded database" do
   before do
     conn = 1234567
     @db = Sequel::Database.new(:single_threaded => true)
-    @db.meta_def(:connect) do |c|
+    meta_def(@db, :connect) do |c|
       conn += 1
     end
   end
@@ -1292,12 +1318,14 @@ describe "A single threaded database" do
   end
   
   specify "should convert an Exception on connection into a DatabaseConnectionError" do
-    db = Sequel::Database.new(:single_threaded => true, :servers=>{}){raise Exception}
+    db = Sequel::Database.new(:single_threaded => true, :servers=>{})
+    def db.connect(*) raise Exception end
     proc {db.pool.hold {|c|}}.should raise_error(Sequel::DatabaseConnectionError)
   end
   
   specify "should raise a DatabaseConnectionError if the connection proc returns nil" do
-    db = Sequel::Database.new(:single_threaded => true, :servers=>{}){nil}
+    db = Sequel.mock(:single_threaded => true, :servers=>{})
+    def db.connect(*) end
     proc {db.pool.hold {|c|}}.should raise_error(Sequel::DatabaseConnectionError)
   end
 end
@@ -1431,19 +1459,24 @@ describe "Database#get" do
   end
   
   specify "should use Dataset#get to get a single value" do
-    @db.get(1).should == 1
-    @db.sqls.should == ['SELECT 1 LIMIT 1']
+    @db.get(:a).should == 1
+    @db.sqls.should == ['SELECT a LIMIT 1']
     
-    @db.get(Sequel.function(:version))
-    @db.sqls.should == ['SELECT version() LIMIT 1']
+    @db.get(Sequel.function(:version).as(:version))
+    @db.sqls.should == ['SELECT version() AS version LIMIT 1']
   end
 
   specify "should accept a block" do
-    @db.get{1}
-    @db.sqls.should == ['SELECT 1 LIMIT 1']
+    @db.get{a}
+    @db.sqls.should == ['SELECT a LIMIT 1']
     
-    @db.get{version(1)}
-    @db.sqls.should == ['SELECT version(1) LIMIT 1']
+    @db.get{version(a).as(version)}
+    @db.sqls.should == ['SELECT version(a) AS version LIMIT 1']
+  end
+
+  specify "should work when an alias cannot be determined" do
+    @db.get(1).should == 1
+    @db.sqls.should == ['SELECT 1 AS v LIMIT 1']
   end
 end
 
@@ -1594,6 +1627,10 @@ describe "Database#each_server with do/jdbc adapter connection string without :a
     end
     hosts.sort.should == [1, 3]
   end
+
+  specify "should raise if not given a block" do
+    proc{Sequel.mock.each_server}.should raise_error(Sequel::Error)
+  end
 end
 
 describe "Database#each_server" do
@@ -1619,7 +1656,7 @@ describe "Database#each_server" do
     @db.each_server do |db|
       dbs << db
       Sequel::DATABASES.should include(db)
-      db.meta_def(:disconnect){dcs << db}
+      meta_def(db, :disconnect){dcs << db}
     end
     dbs.each do |db|
       Sequel::DATABASES.should_not include(db)
@@ -1649,6 +1686,14 @@ describe "Database#raise_error" do
   specify "should convert the exception to a DatabaseDisconnectError if opts[:disconnect] is true" do
     proc{@db.send(:raise_error, Interrupt.new(''), :disconnect=>true)}.should raise_error(Sequel::DatabaseDisconnectError)
   end
+  
+  specify "should convert the exception to an appropriate error if exception message matches regexp" do
+    def @db.database_error_regexps
+      {/foo/ => Sequel::DatabaseDisconnectError, /bar/ => Sequel::ConstraintViolation}
+    end
+    proc{@db.send(:raise_error, Interrupt.new('foo'))}.should raise_error(Sequel::DatabaseDisconnectError)
+    proc{@db.send(:raise_error, Interrupt.new('bar'))}.should raise_error(Sequel::ConstraintViolation)
+  end
 end
 
 describe "Database#typecast_value" do
@@ -1948,6 +1993,12 @@ describe "Database#typecast_value" do
     end
   end
 
+  specify "should raise errors when typecasting hash and array values to String" do
+    [[], {}].each do |i|
+      proc{@db.typecast_value(:string, i)}.should raise_error(Sequel::InvalidValue)
+    end
+  end
+
   specify "should typecast time values to SQLTime" do
     t = Time.now
     st = Sequel::SQLTime.local(t.year, t.month, t.day, 1, 2, 3)
@@ -2018,6 +2069,78 @@ describe "Database#schema_autoincrementing_primary_key?" do
   end
 end
 
+describe "Database#supports_schema_parsing?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_schema_parsing?.should == false
+  end
+
+  specify "should be true if the database implements schema_parse_table" do
+    db = Sequel::Database.new
+    def db.schema_parse_table(*) end
+    db.supports_schema_parsing?.should == true
+  end
+end
+
+describe "Database#supports_foreign_key_parsing?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_foreign_key_parsing?.should == false
+  end
+
+  specify "should be true if the database implements foreign_key_list" do
+    db = Sequel::Database.new
+    def db.foreign_key_list(*) end
+    db.supports_foreign_key_parsing?.should == true
+  end
+end
+
+describe "Database#supports_index_parsing?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_index_parsing?.should == false
+  end
+
+  specify "should be true if the database implements indexes" do
+    db = Sequel::Database.new
+    def db.indexes(*) end
+    db.supports_index_parsing?.should == true
+  end
+end
+
+describe "Database#supports_table_listing?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_table_listing?.should == false
+  end
+
+  specify "should be true if the database implements tables" do
+    db = Sequel::Database.new
+    def db.tables(*) end
+    db.supports_table_listing?.should == true
+  end
+end
+
+describe "Database#supports_view_listing?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_view_listing?.should == false
+  end
+
+  specify "should be true if the database implements views" do
+    db = Sequel::Database.new
+    def db.views(*) end
+    db.supports_view_listing?.should == true
+  end
+end
+
+describe "Database#supports_deferrable_constraints?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_deferrable_constraints?.should == false
+  end
+end
+
+describe "Database#supports_deferrable_foreign_key_constraints?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_deferrable_foreign_key_constraints?.should == false
+  end
+end
+
 describe "Database#supports_transactional_ddl?" do
   specify "should be false by default" do
     Sequel::Database.new.supports_transactional_ddl?.should == false
@@ -2043,8 +2166,8 @@ describe "Database#supports_savepoints_in_prepared_transactions?" do
 
   specify "should be true if both savepoints and prepared transactions are supported" do
     db = Sequel::Database.new
-    db.meta_def(:supports_savepoints?){true}
-    db.meta_def(:supports_prepared_transactions?){true}
+    meta_def(db, :supports_savepoints?){true}
+    meta_def(db, :supports_prepared_transactions?){true}
     db.supports_savepoints_in_prepared_transactions?.should == true
   end
 end
@@ -2166,25 +2289,31 @@ describe "Database extensions" do
   before(:all) do
     class << Sequel
       alias _extension extension
+      remove_method :extension
       def extension(*)
       end
     end
   end
   after(:all) do
     class << Sequel
+      remove_method :extension
       alias extension _extension
+      remove_method :_extension
     end
   end
   before do
     @db = Sequel.mock
   end
+  after do
+    Sequel::Database.instance_variable_set(:@initialize_hook, Proc.new {|db| })
+  end
 
-  specify "should be able to register an extension with a module Database#extension extend the module" do
+  specify "should be able to register an extension with a module have Database#extension extend the module" do
     Sequel::Database.register_extension(:foo, Module.new{def a; 1; end})
     @db.extension(:foo).a.should == 1
   end
 
-  specify "should be able to register an extension with a block and Database#extension call the block" do
+  specify "should be able to register an extension with a block and have Database#extension call the block" do
     @db.quote_identifiers = false
     Sequel::Database.register_extension(:foo){|db| db.quote_identifiers = true}
     @db.extension(:foo).quote_identifiers?.should be_true
@@ -2218,4 +2347,80 @@ describe "Database extensions" do
   specify "should raise an Error if attempting to load an incompatible extension" do
     proc{@db.extension(:foo2)}.should raise_error(Sequel::Error)
   end
+
+  specify "should be able to load an extension into all future Databases with Database.extension" do
+    Sequel::Database.register_extension(:foo, Module.new{def a; 1; end})
+    Sequel::Database.register_extension(:bar, Module.new{def b; 2; end})
+    Sequel::Database.extension(:foo, :bar)
+    @db.should_not respond_to(:a)
+    @db.should_not respond_to(:b)
+    Sequel.mock.a.should == 1
+    Sequel.mock.b.should == 2
+  end
+end
+
+describe "Database specific exception classes" do
+  before do
+    @db = Sequel.mock
+    class << @db
+      attr_accessor :sql_state
+
+      def database_exception_sqlstate(exception, opts={})
+        @sql_state
+      end
+    end
+  end
+
+  specify "should use appropriate exception classes for given SQL states" do
+    @db.fetch = ArgumentError
+    @db.sql_state = '23502'
+    proc{@db.get(:a)}.should raise_error(Sequel::NotNullConstraintViolation)
+    @db.sql_state = '23503'
+    proc{@db.get(:a)}.should raise_error(Sequel::ForeignKeyConstraintViolation)
+    @db.sql_state = '23505'
+    proc{@db.get(:a)}.should raise_error(Sequel::UniqueConstraintViolation)
+    @db.sql_state = '23513'
+    proc{@db.get(:a)}.should raise_error(Sequel::CheckConstraintViolation)
+    @db.sql_state = '40001'
+    proc{@db.get(:a)}.should raise_error(Sequel::SerializationFailure)
+  end
+end
+
+describe "Database.after_initialize" do
+  after do
+    Sequel::Database.instance_variable_set(:@initialize_hook, Proc.new {|db| })
+  end
+
+  specify "should allow a block to be run after each new instance is created" do
+    Sequel::Database.after_initialize{|db| db.sql_log_level = :debug }
+    db = Sequel.mock
+    db.sql_log_level.should == :debug
+  end
+
+  specify "should allow multiple hooks to be registered" do
+    Sequel::Database.after_initialize{|db| db.sql_log_level = :debug }
+    Sequel::Database.after_initialize{|db| db.loggers << 11 }
+
+    db = Sequel.mock
+
+    db.sql_log_level.should == :debug
+    db.loggers.should include(11)
+  end
+
+  specify "should raise an error if registration is called without a block" do
+    proc {
+      Sequel::Database.after_initialize
+    }.should raise_error(Sequel::Error, /must provide block/i)
+  end
+end
+
+describe "Database#schema_type_class" do
+  specify "should return the class or array of classes for the given type symbol" do
+    db = Sequel.mock
+    {:string=>String, :integer=>Integer, :date=>Date, :datetime=>[Time, DateTime],
+      :time=>Sequel::SQLTime, :boolean=>[TrueClass, FalseClass], :float=>Float, :decimal=>BigDecimal,
+      :blob=>Sequel::SQL::Blob}.each do |sym, klass|
+      db.schema_type_class(sym).should == klass
+    end
+  end
 end
